% Function D2TIF converts the plane detector results from MOSE into
% separate TIF image files for each spectral wevelength, which are 
% used in the BLT reconstruction implementation for the SARRP. The
% resulting TIF files are named according to the wavelength of the 
% image (e.g. 590.tif). The number of files outputted depend on the 
% contents of the .D.CW file.
%
% For use with MOSE v.2.3.
% Last Updated: 3/21/2016
% 
% Arguments:
% fullPath: a string containing full path to the input .D.CW file
% outDirectory: a string containing the path to the file output directories
%               (e.g. C:/Users). Omit terminal '/' character needed.

function [] = D2TIF(fullPath, outDirectory)
    DFID = fopen(fullPath);
    i = 1;
    spectrum = [];
    image = [];
    
    C = textscan(DFID, '%s', 1);
    while strcmp(C{1}, 'Spectrum')
        C = textscan(DFID, '%d', 1);
        spectrum = [spectrum C{1}];
        nm = int2str(C{1});
        if isempty(C)
            error('No wavelength specified');
        else
            disp(['Importing virtual detector image at ' nm ' nm.']); 
        end
        C = textscan(DFID, 'TotalDetection %d %f', 1);
        if isempty(C)
            error('No Total Detection specified');
        end
        C = textscan(DFID, 'HeightResolution WidthResolution %d %d', 1);
        if isempty(C)
            error('No Height/Width Resolution specified');
        end
        height = C{1};
        width = C{2};
        C = textscan(DFID, 'Detection Matrix', 1);
        template = repmat('%f ', 1, width);
        C = textscan(DFID, template);
        if size(C{1}, 1) ~= height
            error('Data dimension does not match height specification.');
        end
        layer = flipud(cell2mat(C));
        layermin = min(min(layer));
        layermax = max(max(layer));
        if (layermax ~= 0)
            layer = layer / double(layermax-layermin);
        end
        layer = layer * double(intmax('uint16') - intmin('uint16'));
        image(:,:,i) = layer;
        i = i + 1;
        C = textscan(DFID, '%s', 1);
    end   
    image = uint16(image);
    for j = 1:(i-1)
        imwrite(image(:,:,j), [outDirectory '/' int2str(spectrum(j)) '.tif']);
    end
    fclose(DFID);
end